@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2023, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -22,12 +22,11 @@ log = logging.getLogger(__name__)
 
 # Standard library imports
 import pathlib
-from typing import TYPE_CHECKING, Any as any
+from typing import TYPE_CHECKING, Any
 
 # Bokeh imports
 from ..core.has_props import HasProps, abstract
 from ..core.properties import (
-    Any,
     AnyRef,
     Auto,
     Bool,
@@ -52,9 +51,12 @@ if TYPE_CHECKING:
 
 __all__ = (
     'Callback',
-    'OpenURL',
+    'CloseDialog',
     'CustomJS',
+    'OpenDialog',
+    'OpenURL',
     'SetValue',
+    'ToggleVisibility',
 )
 
 #-----------------------------------------------------------------------------
@@ -68,7 +70,7 @@ class Callback(Model):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
 class OpenURL(Callback):
@@ -77,7 +79,7 @@ class OpenURL(Callback):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     url = String("http://", help="""
@@ -91,11 +93,12 @@ class OpenURL(Callback):
     dependent.
     """)
 
+@abstract
 class CustomCode(Callback):
     """ """
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
 class CustomJS(CustomCode):
@@ -110,7 +113,7 @@ class CustomJS(CustomCode):
     '''
 
     # explicit __init__ to support Init signatures
-    def __init__(self, *args, **kwargs) -> None:
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args, **kwargs)
 
     args = Dict(String, AnyRef)(default={}, help="""
@@ -173,7 +176,7 @@ class CustomJS(CustomCode):
     """)
 
     @classmethod
-    def from_file(cls, path: PathLike, **args: any) -> CustomJS:
+    def from_file(cls, path: PathLike, **args: Any) -> CustomJS:
         """
         Construct a ``CustomJS`` instance from a ``*.js`` or ``*.mjs`` file.
 
@@ -206,7 +209,7 @@ class SetValue(Callback):
     """ Allows to update a property of an object. """
 
     # explicit __init__ to support Init signatures
-    def __init__(self, obj: Init[HasProps] = Intrinsic, attr: Init[str] = Intrinsic, value: Init[any] = Intrinsic, **kwargs) -> None:
+    def __init__(self, obj: Init[HasProps] = Intrinsic, attr: Init[str] = Intrinsic, value: Init[Any] = Intrinsic, **kwargs: Any) -> None:
         super().__init__(obj=obj, attr=attr, value=value, **kwargs)
 
     obj: HasProps = Required(Instance(HasProps), help="""
@@ -217,7 +220,7 @@ class SetValue(Callback):
     The property to modify.
     """)
 
-    value = Required(Any, help="""
+    value = Required(AnyRef, help="""
     The value to set.
     """)
 
@@ -237,6 +240,49 @@ class SetValue(Callback):
         else:
             return f"{self.value!r} is not a valid value for {self.obj}.{self.attr}"
 
+class ToggleVisibility(Callback):
+    """ Toggle visibility of a UI element. """
+
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
+        super().__init__(*args, **kwargs)
+
+    target = Required(Instance(".models.ui.UIElement"), help="""
+    A UI element to show or hide.
+    """)
+
+class OpenDialog(Callback):
+    """ Open a dialog box. """
+
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
+        super().__init__(*args, **kwargs)
+
+    dialog = Required(Instance(".models.ui.Dialog"), help="""
+    A dialog instance to open.
+
+    This will either build and display an new dialog view or re-open an
+    existing dialog view if it was hidden.
+
+    .. note::
+        To display multiple instances of dialog, one needs to clone the
+        dialog's model and use another instance of ``OpenDialog``.
+    """)
+
+class CloseDialog(Callback):
+    """ Close a dialog box. """
+
+    # explicit __init__ to support Init signatures
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
+        super().__init__(*args, **kwargs)
+
+    dialog = Required(Instance(".models.ui.Dialog"), help="""
+    A dialog instance to close.
+
+    The behavior of this action depends on the configuration of the dialog,
+    in particular ``Dialog.close_action`` property.
+    """)
+
 # TODO: class Show(Callback): target = Required(Either(Instance(DOMNode), Instance(UIElement)))
 # TODO: class Hide(Callback): ...
 
