@@ -1,5 +1,5 @@
 #-----------------------------------------------------------------------------
-# Copyright (c) 2012 - 2023, Anaconda, Inc., and Bokeh Contributors.
+# Copyright (c) Anaconda, Inc., and Bokeh Contributors.
 # All rights reserved.
 #
 # The full license is in the file LICENSE.txt, distributed with this software.
@@ -30,10 +30,7 @@ from typing import (
     Any,
     Awaitable,
     Callable,
-    Dict,
-    List,
-    Set,
-    Union,
+    TypeAlias,
 )
 
 # External imports
@@ -45,7 +42,6 @@ from ..core.types import ID
 
 if TYPE_CHECKING:
     from tornado.ioloop import IOLoop
-    from typing_extensions import TypeAlias
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -80,15 +76,15 @@ def fixup_windows_event_loop_policy() -> None:
 
 CallbackSync: TypeAlias = Callable[[], None]
 CallbackAsync: TypeAlias = Callable[[], Awaitable[None]]
-Callback: TypeAlias = Union[CallbackSync, CallbackAsync]
+Callback: TypeAlias = CallbackSync | CallbackAsync
 
-InvokeResult: TypeAlias = Union[Awaitable[None], Awaitable[List[Any]], Awaitable[Dict[Any, Any]]]
+InvokeResult: TypeAlias = Awaitable[None] | Awaitable[list[Any]] | Awaitable[dict[Any, Any]]
 
 Remover: TypeAlias = Callable[[], None]
 
-Removers: TypeAlias = Dict[ID, Remover]
+Removers: TypeAlias = dict[ID, Remover]
 
-RemoversByCallable: TypeAlias = Dict[Callback, Set[ID]]
+RemoversByCallable: TypeAlias = dict[Callback, set[ID]]
 
 class _AsyncPeriodic:
     ''' Like ioloop.PeriodicCallback except the 'func' can be async and return
